import io from 'socket.io-client';
const address: any = import.meta.env.VITE_APP_BASE_API;
const token: any = localStorage.getItem('token');
let myInfo: any = localStorage.getItem('userInfo');
let socket: any = {}; //ws实例对象
let timerManager: any = {}; //计时器管理
let parameter: any = {}; //参数
let testTime: number = 60; //默认时长
let userInfo: any = JSON.parse(myInfo); //用户信息
let beatTime: number = 10000; //心跳频率
let loading: any = null;

export const initWs = (data: any, callback: any) => {
  loading = ElLoading.service({ text: '正在初始化，请稍候', background: 'rgba(0, 0, 0, 0.7)' });
  parameter = data.parameter;
  testTime = data.testTime;
  socket = io(address + '/midexam', {
    transports: ['websocket', 'polling'],
    query: {
      Authorization: token,
      sysuuid: token
    }
  });

  socket.on('connect', (e: any) => {
    getExamStarts();
  });

  socket.on('msg2frontend', (e: any) => {
    callback(e);
    //实时状态
    if (e.cmd === 'exam_status') {
    }
    //工作站状态
    if (e.cmd === 'init_result') {
      if (e.status == 666) {
        parameter.wk_id = e.data.wk_id;
      }
    }
    //测试违规
    if (e.cmd === 'warning_result') {
    }
    //后端播报语音
    if (e.cmd === 'return_audio_msg') {
    }
    //错误提示
    if (e.cmd === 'info_result') {
    }
    //错误提示
    if (e.cmd === 'error_result') {
    }
    //测试中违规提示
    if (e.cmd === 'warning_notify') {
    }
    //断线状态
    if (e.cmd === 'disconnect_request') {
      let message = e.data.message;
      ElMessage({ message: message, type: 'error', duration: 5 * 1000 });
      getExit();
    }
    //状态变更
    if (e.cmd === 'set_exam_state') {
      if (e.data == 3) {
        //关闭遮罩层
        loading?.close();
      }
    }
    //新建测试后返回信息，获取result_id
    if (e.cmd === 'open_one_test_ack') {
    }
    //人脸识别状态
    if (e.cmd === 'face_check_result') {
    }
    //测试结束结果
    if (e.cmd === 'oneresult') {
    }
    //结果生成完成（视频图片）
    if (e.cmd === 'static_urls_finished') {
    }
    //选择学生或测试结束后返回的数据
    if (e.cmd === 'result_info') {
    }
  });

  socket.on('disconnect', (e: any) => {
    callback(e);
    getExit();
  });
};

/**
 * 发送命令
 * @param type发送类型：
 * msgfrom_frontend：测试中命令交互，
 * get_exam_status：心跳，
 * exam_ends：结束测试，
 * fe_reconnect:重连，
 * task_starts：开启工作站，
 * exam_starts：开启测试，
 * join_exam_room：加入房间，
 * @param data发送数据：
 * msgfrom_frontend发送cmd的参：
 * open_one_test:创建测试将由3转40，
 * start_face_recognition:开始人脸识别将由40转41，
 * stop_face_recognition:停止人脸识别将由41转43，
 * face_confirm_only:人脸确认即将测试，
 * start_one_test:开始测试将由43转42，
 * finish_one_test:正在测试中结束并下一次将由42转6转3，
 * close_one_test:中断任何阶段将由42、43、41、40转3，
 * suspend_face_recognition_channels:工作站识别的短跑，某道识别到了人就停止某道的识别，
 * resume_face_recognition_channels:工作站识别的短跑，重新开启某一道的识别，
 * next_test:，
 * result_info:，
 * @param callback回调函数
 */
export const sendMessage = (type: string, data: any, callback?: () => void) => {
  if (socket.connected) {
    callback = callback || function () {};
    socket.emit(type, JSON.stringify(data), callback);
  }
};

/**
 * 连接成功
 */
const getExamStarts = () => {
  sendMessage(
    'exam_starts',
    {
      data: 'start_' + parameter.examId,
      class_id: parameter.classes,
      exam_type: parameter.standard,
      gesture: parameter.gesture,
      test_time: testTime
    },
    () => {
      getNetWork();
    }
  );
};

/**
 * 创建测试
 */
export const openOneTest = () => {
  let examId = parameter.examId;
  sendMessage('msgfrom_frontend', {
    data: {
      cmd: 'open_one_test',
      exam_id: examId
    }
  });
};

/**
 * 开始人脸识别
 */
export const startFace = () => {
  let examId = parameter.examId;
  sendMessage('msgfrom_frontend', {
    data: {
      cmd: 'start_face_recognition',
      exam_id: examId
    }
  });
};

/**
 * 停止人脸识别
 */
export const stopFace = () => {
  let examId = parameter.examId;
  sendMessage('msgfrom_frontend', {
    data: {
      cmd: 'stop_face_recognition',
      exam_id: examId
    }
  });
};

/**
 * 确认并提交人脸
 */
export const faceConfirmOnly = (data: any, callback?: any) => {
  let examId = parameter.examId;
  sendMessage(
    'msgfrom_frontend',
    {
      data: {
        cmd: 'face_confirm_only',
        exam_id: examId,
        result_id: data.result_id,
        student_id: data.student_id,
        gender: data.gender
      }
    },
    () => {
      callback();
    }
  );
};

/**
 * 开始测试
 */
export const startOneTest = (callback?: any) => {
  let examId = parameter.examId;
  sendMessage(
    'msgfrom_frontend',
    {
      data: {
        cmd: 'start_one_test',
        exam_id: examId
      }
    },
    () => {
      callback();
    }
  );
};

/**
 * 停止测试
 */
export const finishOneTest = () => {
  let examId = parameter.examId;
  sendMessage('msgfrom_frontend', {
    data: {
      cmd: 'finish_one_test',
      exam_id: examId
    }
  });
};

/**
 * 关闭测试测试
 */
export const closeOneTest = () => {
  let examId = parameter.examId;
  sendMessage('msgfrom_frontend', {
    data: {
      cmd: 'close_one_test',
      exam_id: examId
    }
  });
};

/**
 * 心跳
 */
const getNetWork = () => {
  timerManager.beat = setInterval(() => {
    let examId = parameter.examId;
    let wk_id = parameter.wk_id;
    sendMessage('get_exam_status', {
      exam_id: examId,
      wk_id: wk_id,
      school_id: userInfo.school_id || null
    });
  }, beatTime);
};

/**
 * 某道识别到了人就停止某道的识别
 */
export const suspendFaceRecognitionChannels = (data: any) => {
  let track = data;
  sendMessage('msgfrom_frontend', {
    data: {
      cmd: 'suspend_face_recognition_channels',
      track: track
    }
  });
};

/**
 * 某道识别到了人就停止某道的识别
 */
export const resumeFaceRecognitionChannels = (data: any) => {
  let track = data;
  sendMessage('msgfrom_frontend', {
    data: {
      cmd: 'resume_face_recognition_channels',
      track: track
    }
  });
};

/**
 * 关闭项目
 */
export const examEnds = () => {
  getExit();
};

/**
 * 退出
 */
const getExit = () => {
  //关闭遮罩层
  loading?.close();
  //清除计时器
  getClearTimer();
  //通知工作站关闭
  sendMessage('exam_ends', {
    data: 'end_' + parameter.examId,
    class_id: parameter.classes
  });
  //如果正在连接就关闭
  if (socket?.connected) {
    socket?.close();
  }
};

/**
 * 清空定时任务
 */
const getClearTimer = () => {
  for (let key in timerManager) {
    if (timerManager.hasOwnProperty(key)) {
      clearInterval(timerManager[key]);
      timerManager[key] = null;
    }
  }
};
